

local MAJOR = "StarLibEvaluator-1.0" 
local MINOR = 1
assert(LibStub, MAJOR.." requires LibStub") 
local LibEvaluator = LibStub:NewLibrary(MAJOR, MINOR)
if not LibEvaluator then return end
local error = LibStub("StarLibError-1.0")
assert(error, "LibEvaluator requires StarLibError-1.0")

if not LibEvaluator.__index then
	LibEvaluator.__index = LibEvaluator
	LibEvaluator.pool = setmetatable({}, {__mode = "k"})
end

LibEvaluator.__call = function(...)
	self.ExecuteCode(...)
end

-- Create a new StarLibEvaluator
-- @name StarLibEvaluator.New
-- @usage LibEvaluator:New()
-- @param errorLevel The errorLevel for this object
-- @return A new StarLibEvaluator object
function LibEvaluator:New(errorLevel) 
	if self ~= LibEvaluator then return end
	
	local obj = next(self.pool)

	if obj then
		self.pool[obj] = nil
	else
		obj = {}
	end

	setmetatable(obj, self)

	obj.errorLevel = errorLevel
	
	return obj	
end

-- Delete a StarLibEvaluator object
-- @name LibEvaluator.Del
-- @usage LibEvaluator:Del(evaluator) or object:Del()
-- @param ev A StarLibEvaluator object
-- @return Nothing
function LibEvaluator:Del(ev)
	if self == LibEvaluator then
		LibProperty.pool[ev] = true
	else
		LibProperty.pool[self] = true
	end
end

do 
	local pool = setmetatable({},{__mode='v'})
	-- Execute some code
	-- @name LibEvaluator.Execute
	-- @usage LibEvaluator.ExecuteCode(self, tag, code, dontSandbox, defval)
	-- @param self The LibEvaluator object or some other enironment. This is the environment for your script.
	-- @param tag A name for your runnable
	-- @param dontSandbox Whether to sandbox the execution or not
	-- @param defval The default value if any
	-- @return ret1, ret2, ret3 -- return values from your code. You can pass 3 return values back from your script.
	LibEvaluator.ExecuteCode = function(self, tag, code, dontSandbox, defval)
		if not defval and not dontDefault then defval = "" end
		
		if not self or not tag or not code then return end

		local runnable = pool[code]
		local err
				
		if not runnable then
			runnable, err = loadstring(code, tag)
			pool[code] = runnable
		end
	
		if not runnable then 
			error:Print(err)
			return nil, err, 0
		end
		
		
		if not dontSandbox then
			local table = StarTip.new()
			table.self = self
			table._G = _G
			table.StarTip = StarTip
			table.select = select
			table.format = format
		
			setfenv(runnable, table)
		
			StarTip.del(table)
		end
		
		local ret1, ret2, ret3 = runnable(xpcall, errorhandler)
		
		defval = loadstring('return ' .. (defval or ""), "defval") or function() return "" end
		
		if not ret1 then 
			ret1 = defval
		end
		
		if type(ret1) == "function" then
			ret1 = ret1()
		end
		
		return ret1, ret2, ret3
	end
end